{% extends "admin/base_site.html" %}

{% load static %}

{% block extrastyle %}
{{ block.super }}
    <link rel="stylesheet" type="text/css" href="{% static 'admin/css/modelfilter.css' %}">
{% endblock %}

{% block extrahead %}
{{ block.super }}
    <script src="{% static 'admin/js/modelfilter.js' %}"></script>
{% endblock extrahead %}
